#! /bin/csh -f
#
#	Auto-convergeing refmac/arp engine
#
alias nawk /usr/bin/nawk

set pdbin      = ./pdb/starthere.pdb
set mtzfile    = /laue/jamesh/keck/mtz/best_phased.mtz

setenv CCP4_SCR   ./temp
set tempfile   = ${CCP4_SCR}/refmactemp
#
set refmacscript = ./scripts/refmac.com
set arpscript    = ./scripts/arp.com
set fftscript    = ./scripts/maps.com
set rmsdscript   = ./scripts/rmsd
#
# scan command line
foreach arg ( $* )
    if($arg =~ *.mtz) set mtzfile = $arg
    if($arg =~ *.pdb) set pdbin = $arg
    if($arg =~ *.brk) set pdbin = $arg
    if(($arg =~ [1-9]*)&&($arg =~ *[0-9])) set cycle = $arg
    if($arg == "noarp") set NO_ARP
    if($arg == nice)  renice -n 40 $$
end
#
# Cycle Numbers for running arp, or quitting 
set cycle     = 0
# FFT will run every "fftcycle" refmac cycles (regardless of convergence)
set fftcycle  = 10
# ARP will run every "arpcycle" refmac cycles (regardless of convergence)
set arpcycle  = 1000
# this script will quit after "maxcycle" total refmac cycles
set maxcycle  = 1000

# get cycle number from input PDB filename
set temp = `echo $pdbin | nawk '{for(i=1;i<length($1);++i){ c=substr($1,i,1); if(c ~ /[0-9]/)printf c }}'`
if("$temp" != "") then
    set cycle = $temp
endif
@ cycle = ( $cycle + 1 )
#
#
#
# make needed directories
if(! -e maps)    mkdir maps
if(! -e logs)    mkdir logs
if(! -e pdb)     mkdir pdb
if(! -e temp)    mkdir temp

REFMAC:
###############################################################################
#
# refine the structure for $NCYC steps
#
# reset convergence variables
set Rslope = 0.1
set RMSxyz = 0.1
set RMSDB  = 0.1
#
echo "refmac cycle $cycle"
./$refmacscript $mtzfile $pdbin >! logs/refmac${cycle}.log
if($status) goto done
set pad_cycle = `echo "$cycle $maxcycle" | nawk '{printf "%0"length($2)"d", $1}'`
mv pdb/refmac.pdb pdb/refmac${pad_cycle}.pdb
#
# evaluate change in the model (rmsCA, rmsXYZ, rmsB, maxXYZ, maxB)
set temp = `./$rmsdscript xlog=on $pdbin pdb/refmac${pad_cycle}.pdb `
if($#temp > 4) then
    set RMSxyz = "$temp[2]"
    set RMSDB  = "$temp[3]"
else
    echo "$0: unable to determine model shifts! "
endif

# prepare for next cycle
set pdbin = pdb/refmac${pad_cycle}.pdb

# check for maximum cycle reached
if($cycle > $maxcycle) goto done

# if refinement is getting nowhere, go to ARP
if("$RMSxyz" =~ 0.000*) set RUN_ARP
#if("$RMSDB" =~ 0.0*) set RUN_ARP
#if(("$RMSDB" =~ 0.0*)&&("$RMSxyz" =~ 0.000*)) set RUN_ARP

if($?RUN_ARP) then
    echo "refmac converged! "
endif

# check cycle counts
if(($cycle % $arpcycle) == 0) set RUN_ARP
if(($cycle % $fftcycle) == 0) goto FFT

if($?RUN_ARP) then
    goto FFT
endif

# repeat cycles of refinement
@ cycle = ( $cycle + 1 )

goto REFMAC




FFT:
###############################################################################
#
# make maps.
#
# defaults to ./maps/2fofc.map ./maps/fofc.map and ./maps/2fofc_free.map 
# plus ./o/2fofc.omap and ./o/fofc.omap 
#
echo "calculating maps"
./$fftscript $pdbin >! ./logs/fft.log
if($status) goto done

if($?RUN_ARP) then
    unset RUN_ARP
    goto ARP
endif

# if we get here, this was just an fft cycle

# repeat cycles of refinement
@ cycle = ( $cycle + 1 )

goto REFMAC



ARP:
if($?NO_ARP) goto done
###############################################################################
#
#   use ARP to add and remove waters
#
echo "running arp"
./$arpscript ./maps/2fofc_free.map ./maps/fofc.map $pdbin >! logs/arpp${cycle}.log
if($status) goto done
mv ./pdb/arp.pdb pdb/arpp${cycle}.pdb

tail -3 logs/arpp${cycle}.log | head -2

# handle convergence loop
set pdbin = pdb/arpp${cycle}.pdb

@ cycle = ( $cycle + 1 )

# edit maps.com if arp complains?

goto REFMAC


done:
###############################################################################
#
#   exiting message
#

echo "REFMAC/ARP stopped!  \07 "
echo ""

exit

