#! /bin/csh -f
#
#
#	Script for automatically retrieving ccp4
#	from the MRC ftp site
#
#
#
# see what kind of system this is
set uname    = `uname | awk '{print tolower($1)}'`
set pwd      = `pwd`
if("$uname" == "IRIX") then
    set uname = `uname | nawk '{print tolower($1)}'`
endif
if("$uname" == "irix") then
    # doesn't support mmcif
    set uname = "$uname --disable-ccif"
endif

set gunzip   = "gunzip"

set ftpsite  = "ftp.ccp4.ac.uk"
set ftpdir   = "ccp4/5.0.2/packed"
set tarfile  = "ccp4-5.0.2.tar.gz"
set dirname  = "ccp4-5.0.2"

if(-e "$1") then
    set tarfile = `basename $1`
    echo "copying $1 to $tarfile"
    cp $1 .
    echo "decompressing $tarfile"
    goto decompress
endif

if(("$1" =~ compile*)||("$2" =~ compile*)) then
    goto compile
endif


retrieve:
# now figure out how we're going to retrieve the file
echo "retrieving $tarfile from ${ftpsite}/$ftpdir"
rm -f $tarfile >& /dev/null
wget --passive-ftp ftp://${ftpsite}/${ftpdir}/$tarfile >& /dev/null
if((! $status)&&(-e $tarfile)) goto decompress

# wget didn't work, try ncftpget
ncftpget ${ftpsite} . ${ftpdir}/$tarfile >& /dev/null
if((! $status)&&(-e $tarfile)) goto decompress

# dangerous! try using regular ftp:
# confirm?
if(-e ~/.netrc) then
    mv ~/.netrc ~/netrc.bak$$
    set RESTORE_NETRC
endif
onintr restore_netrc
echo "default login anonymous password user@site" >! ~/.netrc
chmod go-rwx ~/.netrc

ftp $ftpsite << EOF
get ${ftpdir}/$tarfile $tarfile
close
bye
EOF

restore_netrc:
onintr
if($?RESTORE_NETRC) then
    mv ~/netrc.bak$$ ~/.netrc
else
    rm -f ~/.netrc >& /dev/null
endif

# check if transfer worked
if(! -e "$tarfile") then
    echo "transfer failed..."
    echo "perhaps the server is overloaded? "
    exit 9
endif

decompress:
##########################################################################
set uncompressed = `basename $tarfile .gz`
rm -f $uncompressed >& /dev/null
foreach unzip ( $gunzip unzip "gzip -d" /bin/gunzip /usr/bin/gunzip uncompress )
    $unzip $tarfile >& /dev/null
    if((! $status)&&(-e "$uncompressed")) break
end
untar:
# un-tar the system
tar xf $uncompressed
if($status) then
    echo "unable to decompress archive! "
    echo "is $uncompressed corrupt?"
    exit 9
endif
if(-e "$dirname") then
    cd $dirname
    set pwd = `pwd`
endif

setup:
##########################################################################
# fix up the ccp4.setup file
cat include/ccp4.setup-dist |\
awk '$1=="setenv" && $2=="CCP4_MASTER"{print; $3 = "'`dirname $pwd`'"}\
     $1=="setenv" && $2=="CCP4"       {print; $3 = "$CCP4_MASTER/'`basename $pwd`'"}\
{print}' |\
cat >! include/ccp4.setup

# patches/fixes...


# anything else?

compile:
##########################################################################
# configure the configurator
echo "configuring CCP4...(logging to ${pwd}/compile.log)"
source include/ccp4.setup

# run the configurator
echo "% ./configure $uname --with-x"
./configure $uname --with-x | tee ${pwd}/compile.log
if($status) then
    cat compile.log
    cat << EOF
Sorry, CCP4 build failed.  Please follow the above instructions.
Then type:
cd $pwd
./configure $uname --with-x
make -i
make install
EOF
    exit 9
endif


# now try to build CCP4!  
echo "compiling CCP4...(logging to ${pwd}/compile.log)"
echo "% make -i"
make -i >>& compile.log
if($status) then
    tail -200 compile.log
    echo "Sorry, CCP4 build failed.  Please fix the above problem."
    echo "Then type:"
    echo "make -i"
    echo "make install"
    exit 9
endif
echo "installing CCP4 under ${pwd}"
echo "% make install"
make install      >>& compile.log
if($status) set BAD_COMPILE


# try to compile ancillary/optional programs
ln -sf ../ccp4i/bin/mapslicer ./bin
ln -sf ../ccp4i/bin/loggraph ./bin

# make SURE installation worked
unset BAD
echo "" | bin/mapmask >& /dev/null
if($status > 3) set BAD

if($?BAD) then
    echo "some programs failed to run properly."
    echo "you might want to examine compile.log carefully"
    goto done
endif

echo "SUCCESS! "
goto done

done:
cat << EOF

Before you do anyting else, please fill out the CCP4 license form:
${pwd}/license.txt
or ftp://ccp4a.dl.ac.uk/pub/ccp4/licence.txt

print it out, fill it in, and mail it to:
  Secretary to CCP4        
  Daresbury Laboratory
  Warrington WA4 4AD
  UK

when you have done this, press "Enter"
->
EOF
set temp = ( $< )

cat << EOF
to use CCP4, type:
% source ${pwd}/include/ccp4.setup
  or, put the above command in your .cshrc

to rebuild CCP4:
make whatever changes you want
% cd $pwd
% source include/ccp4.setup
% ./configure $uname --with-x
% make

EOF

exit

