#! /bin/csh -f
#
#	Automatically generated REFMAC script for:
#	 ../wARP/files/warpNtrace.brk and ../mtz/best_phased.mtz
#
###############################################################################
#   Set-up stuff
#
alias nawk /usr/bin/nawk
#
set pdbin      = ./pdb/starthere.pdb
set pdbout     = ./pdb/refmac.pdb
set mtzfile    = ../mtz/best_phased.mtz
set tempfile   = ${CCP4_SCR}/refmactemp
#
# refmac internal cycles to run
set NCYC = 50
#
# scan command line
foreach arg ( $* )
    if($arg =~ *.mtz) set mtzfile = $arg
    if($arg =~ *.pdb) set pdbin = $arg
    if($arg =~ *.brk) set pdbin = $arg
    if(($arg =~ [1-9]*)&&($arg =~ *[0-9])) set NCYC = $arg
    if($arg == "quick") set NCYC = 5
    if($arg == "hurry") set NCYC = 5
    if($arg == nice)  renice -n 40 $$
end
#
# need these directories
if(! -e logs)    mkdir logs
if(! -e pdb)     mkdir pdb
if(! -e temp)    mkdir temp

protin:
###############################################################################
#
#   Run protin  to set up geometric restraints
#
${CCP4_BIN}/protin \
XYZIN      $pdbin \
DICTPROTN  /programs/xtal/ccp4_4.0/lib/data/protin.dic 		\
PROTOUT    ${tempfile}protout.dat		\
PROTCOUNTS ${tempfile}counts.dat		\
<< eof
# automatically determined chains
CHNNAM ID A CHNTYP 1
CHNNAM ID S CHNTYP 2
#capping groups: 5=acetyl, 4=formyl, 3=amino, 2=carboxyl
CHNTYP 1 NTERM 115 THR 3 CTERM 428 GLU 2
CHNTYP 1 CISPEP 1    195      # cis-peptide bonds
CHNTYP 1 SECO  118  129 1    # helix
CHNTYP 1 SECO  118  130 9    # B-bend I 
CHNTYP 1 SECO  123  125 3    # pi helix
CHNTYP 1 SECO  136  144 9    # B-bend I 
CHNTYP 1 SECO  139  143 1    # helix
CHNTYP 1 SECO  152  154 9    # B-bend I 
CHNTYP 1 SECO  178  182 1    # helix
CHNTYP 1 SECO  178  183 9    # B-bend I 
CHNTYP 1 SECO  250  252 9    # B-bend I 
CHNTYP 1 SECO  269  273 1    # helix
CHNTYP 1 SECO  269  274 9    # B-bend I 
CHNTYP 1 SECO  293  295 1    # helix
CHNTYP 1 SECO  293  298 9    # B-bend I 
CHNTYP 1 SECO  314  322 1    # helix
CHNTYP 1 SECO  314  325 9    # B-bend I 
CHNTYP 1 SECO  348  351 9    # B-bend I 
CHNTYP 1 SECO  357  360 1    # helix
CHNTYP 1 SECO  357  361 9    # B-bend I 
CHNTYP 1 SECO  370  373 1    # helix
CHNTYP 1 SECO  370  377 9    # B-bend I 
CHNTYP 1 SECO  387  395 1    # helix
CHNTYP 1 SECO  387  399 9    # B-bend I 
CHNTYP 1 SECO  407  416 1    # helix
CHNTYP 1 SECO  407  417 9    # B-bend I 
CHNTYP 2 WAT

# NCS chains A,B,and C from residues 2-33, 40-77 code 2
# code 1 2 3 4 5 6 
# main t t t m m l
# side t m l m l l
#NONX 3 CHNID A B C NSPANS 2    2 33    2    40 77     2

# of atoms in peptide bond 5 -> Ca to Ca (default)
PEPP 5
SYMM P212121
#VDWRadii 1 CA 7 3.8
# VDW cuttoff distance (5 default)
VDWCUT 5
END
eof
if($status) then
    exit 2
endif
###############################################################################
#
# Refmac step. Refine
#
refmac \
HKLIN      $mtzfile \
HKLOUT     ./refmacout.mtz \
PROTOUT    ${tempfile}protout.dat \
PROTCOUNTS ${tempfile}counts.dat \
PROTSCR    ${tempfile}counts.scr \
XYZIN      $pdbin \
XYZOUT     ${tempfile}refmacout.pdb \
<< END-OF-REFMAC 
LABIN FP=Fhr SIGFP=SIGFhr  FREE=FreeR_flag  HLA=HLADM HLB=HLBDM HLC=HLCDM HLD=HLDDM
LABO FC=FC PHIC=PHIC    FWT=2FOFCWT PHWT=PH2FOFCWT -
                     DELFWT=FOFCWT  PHDELWT=PHFOFCWT
# RESTRAINED, RIGID, or IDEAL (geometry only)
REFI TYPE RESTrained 
# default: refine agains full resolution range
#REFI RESOLUTION  74.536 2.000
# use maximum-likelihood, conjugate-direction refinement (with phases)
REFI RESIDUAL MLKF METHOD CDIR PHASED
# One, overall B (OVERALL) or individual Bs (ISOTROPIC)
REFI BREF ISOTROPIC

#weight of X-ray over geometry (default: WEIG EXPE MATR 0.5)
WEIG EXPE MATR 0.5
# No torsional restraints (combine with low WEIG ? )
#TORSION 0.0
# Bfactor restraints weight
#BFAC 1.0
# NCS restraints weight
#NCSR 1.0

#Blur the phases?
#PHASE SCBL 1 BBLUR 0
# combine experimental phases with calculated ones in output maps
REFI PHASE SIGMAcalc

# Fo-Fc SCALING 
#Scale Fc to Fo with 2-Gaussian fit (TYPE SIMPLE is 1-Gaussian)
SCALE TYPE BULK 
# use an anisotropic, overall B factor (for anisotropic diffraction)
SCALE LSSC ANIS 
# low-res cutoff for scaling (intended for SCALE TYPE SIMPLE)
#SCALE RESO 4.5 2.000

# fix water/protein density ratio to 1/1.35 and water B=80 in scaling
#SCALE LSSC FIXBULK SCBULK -0.74 BBULK 80
# same for SigmaA estimation (NOT recommended)
#SCALE MLSC FIXBULK SCBULK -0.5  BBULK 100

# use free-R hkls in scaling (ignored by default)
#SCALE LSSC FREE
# use experimental sigmas in scaling (ignored by default)
#SCALE LSSC EXPE


# Actual number of REFMAC refinement steps per cycle (5 default)
NCYC $NCYC


# verboseness
MONI FEW
#MONI MEDI HBOND
#MONI MEDI CHIRAL 0.5
#MONI MANY
# 20 display bins by default
#BINS 20
end
END-OF-REFMAC
#
# update the output file
mv ${tempfile}refmacout.pdb $pdbout
#
# Clean up
rm -f ${tempfile}protout.dat
rm -f ${tempfile}counts.dat
rm -f ${tempfile}counts.scr
#
exit


